#ifndef GRAMMAR_H
#define GRAMMAR_H

#include <glib.h>

typedef gint (*nextcb) (gpointer, gpointer*);

typedef struct
{
  gboolean terminal;
  GQuark value;
} symbol_t;

typedef struct _rule rule_t;
typedef struct
{
  GHashTable* grammar;
} grammar_t;

symbol_t* symbol_new (gboolean, GQuark);
symbol_t* symbol_copy (symbol_t*);
guint symbol_hash (gconstpointer);
gboolean symbol_equal (gconstpointer, gconstpointer);
gint symbol_cmp (symbol_t*, symbol_t*);

rule_t* rule_new ();
rule_t* rule_copy (rule_t*);
gint rule_cmp (rule_t*, rule_t*);
gboolean rule_equal (gconstpointer, gconstpointer);
guint rule_hash (gconstpointer);
symbol_t* rule_pop (rule_t*);
void rule_append (rule_t*, symbol_t*);
void rule_delete (rule_t*);

grammar_t* grammar_new ();
rule_t* grammar_rule_new (grammar_t*, symbol_t*);
GList* grammar_get_rules (grammar_*, symbol_t*);
GList* grammar_get_rule (rule_t*);
void grammar_delete (grammar_t*);

#endif
